@@ -23,7 +23,7 @@
 #
 # Standard python modules
 #
-import os,re,string,sys,tempfile,signal
+import os,re,string,sys,tempfile,signal,datetime
 #
 # When writing to a pipe where there is no reader (e.g., when
 # output is directed to head or to less and the user exists from less
@@ -58,6 +58,15 @@ def set_verbosity_level(new_level):
     global verbose
     verbose = new_level
 
+def commify(strnum):
+    strnum = str(strnum)
+    if len(strnum) <= 3:
+        return strnum
+    else:
+        pre = strnum[0:-3]
+        post = strnum[-3:]
+        pre = commify(pre)
+        return pre + "," + post
 #
 # SUPPORT
 #
@@ -74,7 +83,7 @@ def ping_host(hostname):
     #
     elif perform.execute("fping " + hostname + " 2>/dev/null >/dev/null",
                          display=0) != 0:
-    	print "JIG Warning: Could not contact the Debian server at\n" +\
+        print "JIG Warning: Could not contact the Debian server at\n" +\
               "             " + hostname + """
              Perhaps it is down or you are not connected to the network.
              JIG will continue to try to get the information required.
@@ -98,15 +107,15 @@ def do_describe(packages):
     """Print a description of each package.
 
     Arguments:
-    packages	List of packages to describe
+    packages    List of packages (or deb files) to describe
 
     Returns:"""
 
     #
     # From where do we get information about the packages?
     #
-    #   /var/lib/dpkg/available		dpkg's idea of available
-    #	/var/cache/apt/available	apt's  idea of available
+    #   /var/lib/dpkg/available         dpkg's idea of available
+    #   /var/cache/apt/available        apt's  idea of available
     #
     # TODO does apt_pkg provide a way to get available apt-cache directly,
     # and does not require root access - otherwise I need to do it through
@@ -140,6 +149,20 @@ def do_describe(packages):
     # accessible on festiva, alpine, cultus, fairmont. So remove
     # root=1.
     #
+    # First, extract the package names and the package files
+    # and handle separately.
+    #
+    package_files = filter(lambda x: x[-4:] == '.deb', packages)
+    package_names = filter(lambda x: x[-4:] != '.deb', packages)
+    if package_files:
+        for f in package_files:
+            perform.execute("dpkg-deb --info %s" % f)
+            print "="*72
+            sys.stdout.flush()
+    if package_names:
+        packages = package_names
+    else:
+        return
     command = "apt-cache dumpavail"
     packages_pipe = perform.execute(command, noquiet=True, pipe=True)
     avail = apt_pkg.ParseTagFile(packages_pipe)
@@ -238,7 +261,7 @@ def do_download(packages):
     """Download packages.
 
     Arguments:
-    packages	List of packages to be downloaded
+    packages    List of packages to be downloaded
 
     Returns:"""
 
@@ -247,9 +270,7 @@ def do_download(packages):
 
 #------------------------------------------------------------------------
 #
-# FORCE
-#
-# TODO This is being rewritten to handle list of packages - still in progress
+# FORCE : Rewritten 2004-06-05 09:58:47 graham
 #
 #------------------------------------------------------------------------
 def do_force(packages):
@@ -260,30 +281,60 @@ def do_force(packages):
     whatever reason.
 
     Arguments:
-    packages	The packages to be installed (must be in the cache)
+    packages    The packages to be installed (either names or path to deb)
 
     Returns:"""
-
+    #
+    # The basic function is to force install the package using dpkg.
+    #
     command = "dpkg --install --force overwrite --force depends "
+    archives = "/var/cache/apt/archives/"
+    #
+    # For a .deb file we simply force install it.
+    #
     if re.match(".*\.deb$", packages[0]):
         for pkg in packages:
             if os.path.exists(pkg):
-                command = command + "'" + pkg + "' "
-            elif os.path.exists("/var/cache/apt/archives/" + pkg):
-                command = command + "'/var/cache/apt/archives/" + pkg + "' "
+                command += "'" + pkg + "' "
+            elif os.path.exists(archives + pkg):
+                command += "'" + archives + pkg + "' "
             else:
-                print "Wajig: The file " + pkg +\
-                      " not found in either the" +\
-                      " current directory\n" +\
-                      "       nor in /var/cache/apt/archives/\n" +\
-                      "       Please confirm location and try again."
+                print """Wajig: Error: File `%s' not found.
+              Searched current directory and %s.
+              Please confirm the location and try again.""" % (pkg, archives)
                 return()
-        perform.execute(command, root=1)
     else:
+        #
+        # Package names rather than a specific deb package archive
+        # is expected.
+        #
         for pkg in packages:
-            command = command + " /var/cache/apt/archives/$(/bin/ls /var/cache/apt/archives/" +\
-                      " | egrep '^" + pkg + "_' | sort | tail -1)"
-        perform.execute(command,root=1)
+            #
+            # Identify the latest version of the package available in
+            # the download archive, if there is any there.
+            #
+            lscmd  = "/bin/ls " + archives
+            lscmd += " | egrep '^" + pkg + "_' | sort | tail -1"
+            matches = perform.execute(lscmd, pipe=True)
+            debpkg = matches.readline().strip()
+            #
+            # If the package was not found then download it before
+            # it is force installed.
+            #
+            if not debpkg:
+                dlcmd  = "apt-get --quiet=2 --reinstall --download-only "
+                dlcmd += "install '" + pkg + "'"
+                perform.execute(dlcmd, root=1)
+                matches = perform.execute(lscmd, pipe=True)
+                debpkg = matches.readline().strip()
+            #
+            # Force install the package from the download archive.
+            #
+            command += "'" + archives + debpkg + "' "
+    #
+    # The command has been built.  Now execute it.
+    #
+    perform.execute(command, root=1)
 
 #------------------------------------------------------------------------
 #
@@ -294,16 +345,15 @@ def do_hold(packages):
     """Place packages on hold (they will not be upgraded).
 
     Arguments:
-    packages	List of packages.
+    packages    List of packages.
 
     Returns:"""
 
     for p in packages:
-	# The dpkg needs sudo but not the echo.
-	# Maybe sensible to have all sudo's in this file?
-        command = "echo \"" + p + " hold\" | " +\
-	          perform.setroot + " dpkg --set-selections"
-	perform.execute(command)
+        # The dpkg needs sudo but not the echo.
+        # Do all of it as root then!
+        command = "echo \"" + p + " hold\" | dpkg --set-selections"
+        perform.execute(command, root=1)
     print "The following packages are on hold:"
     perform.execute("dpkg --get-selections | egrep 'hold$' | cut -f1")
 
@@ -318,7 +368,7 @@ def do_install(packages):
     """Install packages.
 
     Arguments:
-    packages	List of packages to be installed or path to deb file
+    packages    List of packages to be installed or path to deb file
 
     Returns:"""
 
@@ -399,12 +449,12 @@ def do_install(packages):
 #     elif re.match(".*/+.*", packages[0]):
 #         print "HI"
 #         (packages[0], release) = re.compile(r'/\+').split(packages[0])
-# 	command = "apt-get --target-release %s install %s" %\
+#       command = "apt-get --target-release %s install %s" %\
 #                   (release, perform.concat(packages))
-# 	perform.execute(command, root=1)
+#       perform.execute(command, root=1)
     else:
-	command = "apt-get install " + perform.concat(packages)
-	perform.execute(command, root=1)
+        command = "apt-get install " + perform.concat(packages)
+        perform.execute(command, root=1)
 
 #------------------------------------------------------------------------
 #
@@ -415,7 +465,7 @@ def do_install_suggest(packages, type):
     """Install packages suggested by the listed packages.
 
     Arguments:
-    packages	List of packages 
+    packages    List of packages 
 
     Returns:"""
 
@@ -496,7 +546,7 @@ def do_listinstalled(pattern):
     """Print list of installed packages."""
     command = "dpkg --get-selections | awk '$2 ~/^install$/ {print $1}'"
     if len(pattern) == 1:
-	command = command + " | grep -- " + pattern[0] + " | sort"
+        command = command + " | grep -- " + pattern[0] + " | sort"
     perform.execute(command)
 
 #------------------------------------------------------------------------
@@ -518,7 +568,7 @@ def do_listnames(pattern, pipe=False):
     if len(pattern) == 0:
         command = "apt-cache pkgnames | sort"
     else:
-	command = "apt-cache pkgnames | grep -- " + pattern[0] + " | sort"
+        command = "apt-cache pkgnames | grep -- " + pattern[0] + " | sort"
     return perform.execute(command, root=needsudo, pipe=pipe)
 
 #------------------------------------------------------------------------
@@ -549,7 +599,10 @@ def do_new():
 #
 #------------------------------------------------------------------------
 def map_sources(packages):
-    """Return Source package name for each package in PACKAGES"""
+    """Return Source package name for each package in PACKAGES.
+
+    Note that the source could include a version number.
+"""
     Parse = apt_pkg.ParseTagFile(open("/var/lib/dpkg/available","r"))
     sources = []
     while Parse.Step():
@@ -557,7 +610,9 @@ def map_sources(packages):
         # have been found - to be more efficient
         if Parse.Section.get("Package") in packages:
             if Parse.Section.get("Source"):
-                sources += [Parse.Section.get("Source")]
+                #pkgname = Parse.Section.get("Source").split()[0]
+                pkgname = Parse.Section.get("Source")
+                sources += [pkgname]
             else:
                 sources += [Parse.Section.get("Package")]
     return sources
@@ -566,7 +621,7 @@ def do_news(packages):
     """List latest news for each package.
 
     Arguments:
-    packages	List the latest news for each.
+    packages    List the latest news for each.
 
     Returns: News items
     """
@@ -574,14 +629,21 @@ def do_news(packages):
     results = tempfile.mktemp()
     packages = map_sources(packages)
     for p in packages:
-        # Pipe through Lynx to map HTML entities like &quot;
-        command = "wget --timeout=60 --output-document=-" +\
-                  " http://packages.qa.debian.org/" + p[0] + "/" + p + \
-                  "/news/1.html 2> /dev/null" +\
-                  "| lynx --dump --stdin " +\
-                  " > " + results
-        perform.execute(command)
+        #
+        # Print a header.
+        #
         print "="*((70-len(p))/2) + " " + p + " " + "="*((70-len(p))/2)
+        #
+        # Pipe through Lynx to map HTML entities like &quot;
+        # See comments in do_changelog for details about
+        # source packages with different versions.
+        #
+        command  = "wget --timeout=60 --output-document=-" 
+        command += " http://packages.qa.debian.org/%s/" % (p[0])
+        command += p.split()[0]
+        command += "/news/1.html 2> /dev/null | lynx --dump --stdin > "
+        command += results
+        perform.execute(command)
         command = "cat " + results + " | " +\
                   "awk 'BEGIN{pr=0}/^Description:/{pr=1}/^Files:/{pr=0}" +\
                   "pr==1{print;next}{next}' "
@@ -592,29 +654,75 @@ def do_news(packages):
 #------------------------------------------------------------------------
 #
 # CHANGELOG
+# Updated 2004-06-05 21:05:39 graham
+# Updated 2004-10-19 07:00:08 graham Change host name
 #
 #------------------------------------------------------------------------
 
+def pool_version(package):
+    command = "apt-cache show " + package
+    packages_pipe = perform.execute(command, noquiet=True, pipe=True)
+    Parse = apt_pkg.ParseTagFile(packages_pipe)
+    version = '0'
+    filename = ''
+    while Parse.Step():
+        v = Parse.Section.get("Version")
+        if perform.execute("dpkg --compare-versions %s gt %s" % (v, version))\
+               == 0:
+            version = v
+            filename = Parse.Section.get("Filename")
+    return (filename, version)
+
 def do_changelog(packages):
     """List latest changelog for each package.
 
     Arguments:
-    packages	List the latest changelog for each.
+    packages    List the latest changelog for each.
 
     Returns: Changelogs
     """
-    ping_host("people.debian.org")
+    # (gjw 30 Dec 2004) ping_host("changelogs.debian.net")
+    ping_host("packages.debian.org")
     results = tempfile.mktemp()
     packages = map_sources(packages)
     for p in packages:
-        # Pipe through Lynx to map HTML entities like &quot;
+        #
+        # Print a header.
+        #
         print "="*((70-len(p))/2) + " " + p + " " + "="*((70-len(p))/2)
-        command = "wget --timeout=60 --output-document=-" +\
-                  " http://people.debian.org/~noel/changelogs/pool/main/" +\
-                  p[0] + "/" + p + "/" + p + "_" +\
-                  "`egrep '^" + p + " ' " + available_file +\
-                  " | cut -d' ' -f2`" +\
-                  "/changelog 2> /dev/null"
+        #
+        # Some source pakages have a different version which will
+        # be included in the string: e.g. "bash (2.05b-2-15)".
+        # We handle this case by using this version number rather than
+        # the packages installed version number.
+        #
+        pkg = p.split()
+        if len(pkg) > 1:
+            ver = pkg[1][1:-1] # Remove the round brackets
+            pkg = pkg[0]
+        else:
+            pkg = pkg[0]
+            ver = "`egrep '^%s ' %s | cut -d' ' -f2`" % (pkg, available_file)
+        #
+        # Find pool file name and get latest version number.
+        # Previously I relied on changelogs.debian.net, but it seems that
+        # the symlink from, e.g., http://changelogs.debian.net/metacity
+        # to the actual changelog file does not get updated very quickly.
+        # So move to pointing the to latest version's actual changelog
+        # on packages.debian.org. (gjw 30 Dec 2004).
+        #
+        (pool, version) = pool_version(pkg)
+        changelog_path = re.sub(r'_[^_]*\.deb$', '/changelog', pool)
+        #
+        # Generate the command to retireve the changelog.
+        #
+        command  = "wget --timeout=60 --output-document=-"
+        # (gjw 30 Dec 2004) command += " http://changelogs.debian.net/"
+        command += " http://packages.debian.org/changelogs/"
+        #command += p[0] + "/"
+        #command += "%s/%s_%s/changelog 2> /dev/null" % (pkg, pkg, ver)
+        # (gjw 30 Dec 2004) command += "%s 2> /dev/null" % (pkg)
+        command += "%s 2> /dev/null" % (changelog_path)
         perform.execute(command)
 
 #------------------------------------------------------------------------
@@ -702,7 +810,8 @@ def do_size(packages, size):
     # Print out the one line descriptions.
     #
     pkgs = size_list.keys()
-    pkgs.sort()
+    # pkgs.sort()
+    pkgs.sort(lambda x, y: cmp(int(size_list[x]), int(size_list[y])))
     #
     # Output
     #
@@ -712,27 +821,28 @@ def do_size(packages, size):
         print "%-30s %s       %s" % ("Package", "Size", "Status")
         print "="*30 + "-" + "="*10 + "-" + "="*35
         for pkg in pkgs:
-            print "%-30s %6s     %-20s" \
-                  % (pkg, size_list[pkg], status_list[pkg])
+            print "%-30s %10s     %-20s" \
+                  % (pkg, commify(size_list[pkg]), status_list[pkg])
 
 #------------------------------------------------------------------------
 #
 # STATUS
 #
 #------------------------------------------------------------------------
-def do_status(packages):
+def do_status(packages, snapshot=False):
     """List status of the packages identified.
 
     Arguments:
-    packages	List the version of installed packages
+    packages    List the version of installed packages
+    snapshot    Whether a snapshot is required (affects output format)
 
     Returns:"""
 
-    print "%-23s %-15s %-15s %-15s %s" %\
-          ("Package", "Installed", "Previous", "Now", "State")
-    print "="*23 + "-" + "="*15 + "-" + "="*15 + "-" + "="*15 + "-" + "="*5
-    sys.stdout.flush()
-
+    if not snapshot:
+        print "%-23s %-15s %-15s %-15s %s" %\
+              ("Package", "Installed", "Previous", "Now", "State")
+        print "="*23 + "-" + "="*15 + "-" + "="*15 + "-" + "="*15 + "-" + "="*5
+        sys.stdout.flush()
     #
     # Get status.  Previously used dpkg --list but this truncates package
     # names to 16 characters :-(. Perhaps should now also remove the DS
@@ -752,9 +862,9 @@ def do_status(packages):
     #
     command = "dpkg --get-selections | join - " + ifile + " | " +\
               "join -a 1 - " + previous_file + " | " +\
-	      "awk 'NF==3 {print $0, \"N/A\"; next}{print}' | " +\
-	      "join -a 1 - " + available_file + " | " +\
-	      "awk 'NF==4 {print $0, \"N/A\"; next}{print}' | "
+              "awk 'NF==3 {print $0, \"N/A\"; next}{print}' | " +\
+              "join -a 1 - " + available_file + " | " +\
+              "awk 'NF==4 {print $0, \"N/A\"; next}{print}' | "
     if len(packages) > 0:
         # Use grep, not egrep, otherwise g++ gets lost, for example!
         command = command + "grep '^\($"
@@ -763,9 +873,14 @@ def do_status(packages):
         command = command + " \)' |"
 
     command = command +\
-	      "awk '{printf(\"%-20s\\t%-15s\\t%-15s\\t%-15s\\t%-2s\\n\", " +\
-	      "$1, $3, $4, $5, $2)}'"
-    perform.execute(command)
+              "awk '{printf(\"%-20s\\t%-15s\\t%-15s\\t%-15s\\t%-2s\\n\", " +\
+              "$1, $3, $4, $5, $2)}'"
+    if snapshot:
+        fobj = perform.execute(command, pipe=True)
+        for l in fobj:
+            print "=".join(l.split()[0:2])
+    else:
+        perform.execute(command)
     #
     # Check whether the package is not in the installed list, and if not
     # list its status appropriately.
@@ -773,16 +888,16 @@ def do_status(packages):
     for i in packages: 
         if os.system("egrep '^" + i + " ' " + ifile + " >/dev/null"):
             #
-	    # Package is not installed.
-	    #
-	    command = \
+            # Package is not installed.
+            #
+            command = \
               "join -a 2 " + previous_file + " " + available_file + " | " +\
-	      "awk 'NF==2 {print $1, \"N/A\", $2; next}{print}' | " +\
-	      "egrep '^" + i + " '"
-	    command = command +\
-	      " | awk '{printf(\"%-20s\\t%-15s\\t%-15s\\t%-15s\\n\", " +\
-	      "$1, \"N/A\", $2, $3)}'"
-	    perform.execute(command)
+              "awk 'NF==2 {print $1, \"N/A\", $2; next}{print}' | " +\
+              "egrep '^" + i + " '"
+            command = command +\
+              " | awk '{printf(\"%-20s\\t%-15s\\t%-15s\\t%-15s\\n\", " +\
+              "$1, \"N/A\", $2, $3)}'"
+            perform.execute(command)
     #
     # Tidy up - remove the "installed file"
     #
@@ -836,10 +951,9 @@ def do_unhold(packages):
 
     for p in packages:
         # The dpkg needs sudo but not the echo.
-        # Maybe sensible to have all sudo's in this file?
-        command = "echo \"" + p + " install\" | " +\
-                  perform.setroot + " dpkg --set-selections"
-        perform.execute(command)
+        # Do all of it as root then.
+        command = "echo \"" + p + " install\" | dpkg --set-selections"
+        perform.execute(command, root=1)
     print "The following packages are still on hold:"
     perform.execute("dpkg --get-selections | egrep 'hold$' | cut -f1")
         
@@ -852,7 +966,7 @@ def do_update(query=0, quiet=0):
     """Perform.Execute an update of the available packages
 
     Arguments:
-    query	If non-zero then ask user whether this should be done.
+    query       If non-zero then ask user whether this should be done.
 
     Returns:
     Return status of the command"""
@@ -860,21 +974,21 @@ def do_update(query=0, quiet=0):
     if verbose > 0:
         print "The Packages files listing the available packages is being updated."
     if query > 0:
-	versionfile=open("/etc/debian_version");
-	debian_version = string.split(versionfile.read())[0];
-	if debian_version == "testing/unstable":
-	    print """
+        versionfile=open("/etc/debian_version");
+        debian_version = string.split(versionfile.read())[0];
+        if debian_version == "testing/unstable":
+            print """
 You appear to be running the `unstable' or `testing' distribution of
 Debian. It is very likely that your Packages files are out of date. 
 Doing an UPDATE is recommended.
 """
-	else:
-	    print """
+        else:
+            print """
 You appear to be running the `stable' distribution of Debian. It is not 
 likely that your Packages files are out of date. Doing an UPDATE is optional
 """
-	doit = raw_input("Update? [Y/n] ")
-	if not re.match("^(y|Y|yes|Yes|YES)$", doit):
+        doit = raw_input("Update? [Y/n] ")
+        if not re.match("^(y|Y|yes|Yes|YES)$", doit):
             return
     #
     # Note that we do a "dselect update" rather than an "apt-get update" since the
@@ -896,13 +1010,13 @@ likely that your Packages files are out of date. Doing an UPDATE is optional
         quietopt = ""
     if perform.execute("dselect update " + quietopt, root=1) == 0:
     # if perform.execute("apt-get update", root=1) == 0:
-	#
-	# Only update the available list if the UPDATE succeeded
- 	#
+        #
+        # Only update the available list if the UPDATE succeeded
+        #
         changes.update_available()
-	#
-	# How many new upgrades are there?
-	#
+        #
+        # How many new upgrades are there?
+        #
         print "There are " + changes.count_upgrades() + " new upgrades"
     else:
         print "Wajig: The update failed for some reason."
@@ -923,7 +1037,7 @@ def do_whatis0(packages):
     """Look for a one-line description of the given packages.
 
     Arguments:
-    packages	List of packages to get descriptions for
+    packages    List of packages to get descriptions for
 
     Returns:"""
 
@@ -931,20 +1045,20 @@ def do_whatis0(packages):
     if perform.execute(command) == 0:
         for i in packages:
             results = tempfile.mktemp()
-	    command = "wget --output-document=" + results +\
-		      " http://packages.debian.org/cgi-bin" +\
-		      "/search_packages.pl\?keywords=" + i +\
-		      "\&searchon=names\&subword=1\&" +\
-		      "version=unstable\&release=all 2> /dev/null"
-	    perform.execute(command)
-	    command = "cat " + results + " | " +\
-		"egrep '<TD><B><A HREF|<TD COLSPAN=2>' | " +\
-		"perl -p -e 's|<[^>]*>||g;s|^	 ||g;" +\
-	        "s|^&nbsp; |	|;s|&nbsp;||g;s|&quot;|\"|g;'"
-	    perform.execute(command)
-	    if os.path.exists(results): os.remove(results)
+            command = "wget --output-document=" + results +\
+                      " http://packages.debian.org/cgi-bin" +\
+                      "/search_packages.pl\?keywords=" + i +\
+                      "\&searchon=names\&subword=1\&" +\
+                      "version=unstable\&release=all 2> /dev/null"
+            perform.execute(command)
+            command = "cat " + results + " | " +\
+                "egrep '<TD><B><A HREF|<TD COLSPAN=2>' | " +\
+                "perl -p -e 's|<[^>]*>||g;s|^    ||g;" +\
+                "s|^&nbsp; |    |;s|&nbsp;||g;s|&quot;|\"|g;'"
+            perform.execute(command)
+            if os.path.exists(results): os.remove(results)
     else:
-	print "Perhaps the server is temporarily down."
+        print "Perhaps the server is temporarily down."
 
 #------------------------------------------------------------------------
 #
@@ -955,7 +1069,7 @@ def do_whichpkg(filename):
     """Look for a particular file or pattern in a Debian package.
 
     Arguments:
-    filename	Pattern to find
+    filename    Pattern to find
 
     Returns:"""
 
@@ -1046,7 +1160,7 @@ def do_findpkg(pkg):
     """Look for a particular pkg at apt-get.org.
 
     Arguments:
-    pkg		Package to find
+    pkg         Package to find
 
     Returns:"""
     ping_host("www.apt-get.org")
@@ -1148,7 +1262,7 @@ def do_recdownload(packages):
     perform.execute(command, root=1)
 
 def main():
-    do_size([])
+    do_size([], 100)
 
 if __name__ == '__main__':
     main()
