@@ -47,6 +47,7 @@ if os.path.exists("/usr/bin/sudo"):
 quiet = ""
 
 def set_quiet():
+    global quiet
     quiet = "> /dev/null"
 
 simulate = 0
@@ -81,10 +82,10 @@ def execute(command, root=0, noquiet=0, display=1, pipe=0):
 
     Arguments:
     
-    COMMAND	A string containing the command and command line options
-    ROOT	If non-zero then root access is required to execute command
-    NOQUIET	Suppress the use of quiet (in case command has a redirect)
-    PIPE	If True then return a file-like object.
+    COMMAND     A string containing the command and command line options
+    ROOT        If non-zero then root access is required to execute command
+    NOQUIET     Suppress the use of quiet (in case command has a redirect)
+    PIPE        If True then return a file-like object.
 
     Returns:
     
@@ -99,19 +100,17 @@ def execute(command, root=0, noquiet=0, display=1, pipe=0):
     elif simulate > 0 and display == 1:
         print command
     if root > 0:
-	if setroot == "/usr/bin/sudo":
- 	    command = setroot + " " + command
+        if setroot == "/usr/bin/sudo":
+            # If the command contains a pipe, and requires root,
+            # each part must be sudo.
+            command = setroot + " " + command.replace("|", "| %s " % setroot)
         else:
-	    if quiet == "" and os.getenv("USER") != "root":
-		print """
-Using `su' but please consider installing `sudo' which is more secure.
-With `sudo' users type their own password and do not need to know the
-root password. Also, the password is remembered for a short while so
-that you won't be asked for it again while interacting with wajig.
-Have a look at the wajig documentation (wajig doc) for a quick guide
-to configuring /etc/sudoers.
+            if quiet == "" and os.getenv("USER") != "root":
+                print """
+Using `su' and requiring root password. Install `sudo' to support user
+passwords. See wajig documentation (wajig doc) for details.
 """
-            command = setroot + " -c \"" + command + "\""
+            command = setroot + " -c '" + command + "'"
     if noquiet > 0:
       command = command
     else:
